import java.io.*;

/**
*   The standard OLine object
*	@author		K. Zouba
*	@version	05/2005
*/
public class OLine extends OGraphicImpl {

	private OPoint b,e;

	/** Default constructor */
	public OLine() {
		b = new OPoint();
		e = new OPoint();
		}

	/** Constructor with int */
	public OLine(double x1,double y1, double x2,double y2) {
		b= new OPoint(x1,y1);
		e= new OPoint(x2,y2);
		}

	/** Constructor with point */
	public OLine (OPoint b, OPoint e) {
		this.b = (OPoint)b.copy(); this.e = (OPoint)e.copy();
		}

	public OGraphic copy()
		{return new OLine((OPoint)b.copy(),(OPoint)e.copy());}

	/** getDX() */
	public double getDX() {
		return e.getX()-b.getX();
		}

	/** getDY() */
	public double getDY() {
			return e.getY()-b.getY();
		}

	/** isParallel() */
	public boolean isParallel(OLine l) {
	if((this.getDX()*l.getDY() - this.getDY()*l.getDX()) == 0)
				return true;
			return false;
	}

	/** getBegin() */
	public OPoint getBegin() {
		return b;
	}

	/** getEnd() */
	public OPoint getEnd() {
		return e;
	}

	/**
	*  Get line's direction in radian,
	*  the arctangent of (y2-y1)/(x2-x1)
	*/
	public double getDirection()
	{
		return Math.atan2(getDY(),getDX());
	}



	/**
	*	The point is between line's begin and line's end
	*	dx1 dx2 are opposite or dy1 dy2 must be opposite
	*	This method isn't bijective
	*/
	public boolean between (OPoint p ) {
		double dx1,dx2,dy1,dy2;
		dx1 = p.getDX(b); dy1 = p.getDY(b);
		dx2 = p.getDX(e); dy2 = p.getDY(e);
		if((dx1*dx2 < 0)||(dy1*dy2 < 0))
		return true;
		return false;
		}

	/**
	*  Move the line with dx,dy
	*  mode 1 move begin and end, 2 begin, 3 end
	*/
	public void move(double dx,double dy,int mode) {
		switch(mode)
			{
			case 1:
				b.move(dx,dy);
				e.move(dx,dy);
				break;
			case 2:
				b.move(dx,dy);
				break;
			case 3:
				e.move(dx,dy);
				break;
			}
		}

	/**
	*  The lines overlap
	*/
	public boolean overlap (OLine l ) {
		if
			(
			(this.between(l.getBegin()))
			||
			(this.between(l.getEnd()))
			)
			return true;
		return false;
		}

	/**
	*  Get tangent of line (y2-y1)/(x2-x1)
	*/
	public double getTangent() {
		return ((double)getDY()/(double)getDX());
		}

	/**
	*  Get origin
	*/
	public double getOrigin() {
		return (Math.round(b.getY() - this.getTangent()*b.getX()));
		}

	/**
	*  Test if a point corresponds to line equation y=m*x + p
	*/
	public boolean isColinear (OPoint p ) {
		double ay = this.getTangent()*p.getX() + this.getOrigin();
		if(p.getY() == Math.round(ay))
			return true;
		return false;
		}

	/**
	*  Get colinear intersection between two line objects : junction detection
	*/
	public OPoint getIntersection( OLine l) {
		OPoint pi = new OPoint();
		if(!this.isParallel(l)) {
			double m1,p1,m2,p2;
			m1 = this.getTangent(); p1 = this.getOrigin();
			m2 = l.getTangent(); p2 = l.getOrigin();
			int xi = (int)(Math.round((p2-p1)/(m1-m2)));
			int yi = (int)(Math.round((m2*p1-p2*m1)/(m2-m1)));
			if((xi>=0)&&(yi>=0))
			pi.set(xi,yi);
			}
		return pi;
		}

	/**
	*	The lines are orthogonal
	*/
	public boolean isOrthogonal ( OLine l ) {

		if((this.getDX()*l.getDX() + this.getDY()*l.getDY()) == 0)
			return true;
		return false;
		}

	/**
	*  Get Euclidien distance between the line object and a given point
	*/
	public double euclid (OPoint p ) {

		double num = Math.abs(p.getY() - (this.getTangent()*p.getX()) - this.getOrigin());
		double den = Math.sqrt(1+Math.pow(getTangent(),2));
		return(num/den);
		}

	/**
	*  Get length of line
	*/
	public double getLength() {
		return e.euclid(b);
		}

	public double getAngle( OLine l) {

			double angle=-1;
			int choix=0;
			if(this.isOrthogonal(l)) choix=1;
			if(this.isParallel(l)) choix=2;
			if(choix==2) return Math.PI;
			if((choix==1)||(choix==0)) {
				if(e.equals(l.getBegin())) { return this.getRAngle(l);}
				if(b.equals(l.getEnd())) {return this.getReverse().getRAngle(l.getReverse());}
				if(b.equals(l.getBegin())) { return this.getReverse().getRAngle(l);}
				if(e.equals(l.getEnd())) { return this.getRAngle(l.getReverse());}
			}
			return angle;
	}

	/**
	* Get minimum Direct angle between the line
	* and a end connected line
	**/

	public double getDAngle( OLine l2) {

	if(!e.equals( l2.getBegin()))
		return -1;

		OLine l3= new OLine(b,l2.getEnd());
		double a,b,c;
		a = this.getLength();
		b = l2.getLength();
		c = l3.getLength();
		double r = Math.acos((Math.pow(a,2) + Math.pow(b,2) - Math.pow(c,2))/(2*a*b));
		//r=r*180/3.1415957; in degree
		return r;
		}

	/**
	*  Get Direct angle between two acute angles
	*  or right angles.
	**/

		public double getRAngle(OLine l) {

			double x = this.getDirection();
			double y=l.getDirection();
			double diff=x-y;
			double angle=this.getDAngle(l);
			double angleinv=2*Math.PI-angle;
			if(diff>Math.PI) diff=diff-2*Math.PI;
			if(diff<-Math.PI) diff=2*Math.PI+diff;
			if(diff>0) return angle;
			else return angleinv;
		}


	/** equals Method */
	public boolean equals (OLine l) {

		if((this.b.equals(l.b)) &&(this.e.equals(l.e))) return true;
		return false;
		}

	/**
	*  Test if lines are connected,
	*  they are a same point (begin or end)
	*/
	public boolean isConnected ( OLine l ) {
		if	(
			(this.getBegin().equals(l.getBegin()))
			||
			(this.getBegin().equals(l.getEnd()))
			||
			(this.getEnd().equals(l.getBegin()))
			||
			(this.getEnd().equals(l.getEnd()))
			)
			return true;
		return false;
		}

	/**
	*  Test if lines are connected,
	*  they are a same point (begin)
	*/
	public boolean isConnected_begin ( OLine l ) {
			if	(
				(this.getBegin().equals(l.getBegin()))
				||
				(this.getEnd().equals(l.getBegin()))
				)
				return true;
			return false;
		}

	/**
	*  Test if lines are connected,
	*  they are a same point (end)
	*/
	public boolean isConnected_end ( OLine l ) {
				if	(
					(this.getBegin().equals(l.getEnd()))
					||
					(this.getEnd().equals(l.getEnd()))
					)
					return true;
				return false;
		}


	/** Method movec **/
	public void movec() {
			OPoint p=b;
			b=b.minus(b);
			e=e.minus(p);
		}

	/** gomWrite **/

	public void gomWrite(BufferedWriter flow) {
		try {
			flow.write(getName()+"\n");
			b.gomWrite(flow);
			e.gomWrite(flow);
			}
		catch(IOException e) {System.out.println("IOException");}
		}

	/**
	*  Reverse and return a new line
	*  (end to begin and begin to end)
	*/
	public  OLine getReverse() {

		OLine rl=new OLine(this.e,this.b);
		return rl;
}
	/**
	* test if a line incorporates another line
	*/
	public boolean incorporates(OLine l) {
		OPoint begin=l.getBegin();
		OPoint end=l.getEnd();
		if((this.isColinear(begin))||(this.isColinear(end)))
		if((this.between(begin))||(this.between(end)))
		return true;
		return false;
	}

	/** xmlWrite **/
	public void xmlWrite(BufferedWriter flow) {
		try {
			flow.write("<" + getName() + ">\n");
			b.xmlWrite(flow);
			e.xmlWrite(flow);
			flow.write("</" + getName() + ">\n");
			}
		catch(IOException e) {System.out.println("IOException");}
		}

	/** svgWrite */
	public void svgWrite(BufferedWriter flow) {
		try {
			flow.write("<line x1=\""+b.getX()+"\" y1=\""+b.getY()+"\" x2=\""+e.getX()+"\" y2=\""+e.getY()+"\" style=\"fill:none; stroke:green; stroke-width:1\"/>\n");
			}
		catch(IOException e) {System.out.println("IOException");}
		}

	/** gomRead **/
	public void gomRead(BufferedReader flow) {
		String gName;
		try {
			gName = flow.readLine() ;
			b.gomRead(flow);
			gName = flow.readLine() ;
			e.gomRead(flow);
			}
		catch(IOException e) {System.out.println("IOException");}
		}
	/** This method writes in a xgmml format **/
	public void xgmmlWrite (BufferedWriter flow) {}

	}