import java.io.*;


/**
*   The standard OPoint object
*	@author		K. Zouba
*	@version	05/2005
*/

public class OPoint extends OGraphicImpl {

 	private double x=0, y=0;

 	/**  Constructor without args */
 	public OPoint()	{
 		x = y = -1;
 		}

 	/**  Constructor with 2 args */
 	public OPoint(double x,double y) {
 		this.x = x;
 		this.y = y;
 		}

	public OGraphic copy()
		{return new OPoint(x,y);}

	/**  Clear the x;y values */
	public void clear()
		{ x = -1; y = -1; }

	/** Set the x;y values */
	public void set(double x,double y)
		{ this.x = x; this.y = y; }

	/** Set the x value */
	public void setX(double x)
		{ this.x = x;}

	/** Set the y value */
	public void setY(double y)
		{ this.y = y;}

	/** Get the x value */
	public double getX()
		{return x;}

	/** Get the y value */
	public double getY()
		{return y;}

 	/**   Get the p(x) - this(x) delta x */
 	public double getDX (OPoint p) {
 		return p.getX()-x;
 		}

 	/**
 	/**  Get the p(y) - this(y) delta y */
 	public double getDY (OPoint p) {
 		return p.getY()-y;
 		}


 	/**  Euclid */
 	public double euclid (OPoint p) {
 		return Math.sqrt(Math.pow(x-p.getX(),2) + Math.pow(y-p.getY(),2));
 		}

	/**  Move */
	void move(double dx,double dy) {
		x += dx ;
		y += dy ;
		}

	/**  Scalar */
	void scalar(double a) {
		OPoint c = new OPoint(0,0);
		double l = euclid(c);
		x = (int)Math.round(l*Math.cos(a));
		y = y - (int)Math.round(l*Math.sin(a));
		}

	/** Rotat */
	void rotat(double a,double length) {

		if(a!=0) {
		x=(int)(length*Math.cos(a));
		y=(int)(-y+length*Math.sin(a));
		}
	}


	/**  Get following according  to the Freeman direction (0 to 7) */
	OPoint getFollowing(int d) {
		OPoint p = new OPoint();
		switch(d) {
			case 0:
				p.set(x+1,y);
				break;
			case 1:
				p.set(x+1,y-1);
				break;
			case 2:
				p.set(x,y-1);
				break;
			case 3:
				p.set(x-1,y-1);
				break;
			case 4:
				p.set(x-1,y);
				break;
			case 5:
				p.set(x-1,y+1);
				break;
			case 6:
				p.set(x,y+1);
				break;
			case 7:
				p.set(x+1,y+1);
				break;
			}
		return p;
		}

 	/** Equals Method */
 	public boolean equals(OPoint p) {
 		if((this.x==p.x)&&(this.y==p.y)) return true;
 		return false;
 		}

 	/** Minus Method */
 	public OPoint minus(OPoint p ) {

		double xr=x-p.getX();
		double yr=y-p.getY();
		OPoint r= new OPoint(xr,yr);
		return r;
	}


 	public void gomWrite(BufferedWriter flow) {
 		try {
 			flow.write(getName()+"\n");
 			flow.write(String.valueOf(x)+"\n");
 			flow.write(String.valueOf(y)+"\n");
 			}
 		catch(IOException e) {System.out.println("IOException");}
 		}

 	public void xmlWrite(BufferedWriter flow) {
 		try
 			{flow.write("<"+ getName()+ " x=\""+x+"\" y=\""+y+"\"/>\n");}
 		catch(IOException e) {System.out.println("IOException");}
 		}

 	public void svgWrite(BufferedWriter flow) {
 		try {flow.write("<line x1=\""+x+"\" y1=\""+y+"\" x2=\""+(x+1)+"\" y2=\""+y+"\" style=\"fill:none; stroke:blue; stroke-width:1\"/>\n");}
 		catch(IOException e) {System.out.println("IOException");}
 		}

 	public void gomRead(BufferedReader flow) {
 		try {
 			x = Double.parseDouble(flow.readLine());
 			y = Double.parseDouble(flow.readLine());
 			}
 		catch(IOException e) {System.out.println("IOException");}
 		}
 	public void xgmmlWrite (BufferedWriter flow) {}
	}